#!/bin/bash -e
#
# install-api:install-api.sh
#
if [ "$( type -t echofatal )" != 'function' ] ; then
  echo 'https://git.cclaude.rocks/bash-apis/bash-common-api is required: Function echofatal not found.' >&2
  exit 100
fi
if [ "$( type -t exit_if_variable_not_define_or_empty )" != 'function' ] ; then
  echofatal 'https://git.cclaude.rocks/bash-apis/bash-common-api is required Function exit_if_variable_not_define_or_empty not found.' >&2
  exit 100
fi

function _get_package_manager_ {
  local pkg_manager="$1" # R/W

  if [ -z "${pkg_manager}" ] ; then
    pkg_manager="$( command -v apt-get || command -v pkg || command -v yum )"
  fi

  if [ -z "${pkg_manager}" ] ; then
    echofatal 'Package manager not found.' >&2
    exit 100
  fi
  echo "${pkg_manager}"
}

function install_update {
  local pkg_manager=
  pkg_manager="$( _get_package_manager_ "$1" )"
  if [ -z "${pkg_manager}" ] ; then
    exit 101
  fi

  "${pkg_manager}" update || exit $?
}

function install_install_if_needed {
  local pkg_manager=
  pkg_manager="$( _get_package_manager_ "$1" )"
  if [ -z "${pkg_manager}" ] ; then
    exit 101
  fi
  shift

  local command_name=
  local package_name=

  while [ "$#" -gt 0 ] ; do
    IFS=";" read -r command_name package_name <<< "${1}"
    shift

    if [ -z "${package_name}" ] ; then
      package_name="${command_name}"
    fi

    if ! command -v "${command_name}" >/dev/null ; then
      "${pkg_manager}" install -y "${package_name}" || exit $?
    fi
  done
}

function install_if_needed { # DEPRECATED
  local -r skip_update="$1"
  local    pkg_manager="$2" # R/W
  shift
  shift

  echofatal 'install_if_needed is deprecated, use install_update and install_install_if_needed instead'

  if [ -z "${pkg_manager}" ] ; then
    pkg_manager="$( command -v apt-get || command -v pkg || command -v yum )"
  fi

  if [ -z "${pkg_manager}" ] ; then
    echofatal 'Package manager not found.' >&2
    exit 1
  fi

  if [ "${skip_update}" != 'skip_update' ] ; then
    "${pkg_manager}" update  || exit $?
  fi

  local command_name=
  local package_name=

  while [ "$#" -gt 0 ] ; do
    IFS=";" read -r command_name package_name <<< "${1}"
    shift

    if [ -z "${package_name}" ] ; then
      package_name="${command_name}"
    fi

    if ! command -v "${command_name}" >/dev/null ; then
      "${pkg_manager}" install -y "${package_name}" || exit $?
    fi
  done
}

function install_set_chmod_for_list {
  local -r mode="$1"
  local -r files="$2"

  local file=
  while read -r file ; do
    if [ -n "${file}" ] ; then
      chmod -v "${mode}" "${INSTALL_DATA_DIR}/${file}" || exit $?
    fi
  done <<<"${files}"
}

function install_links {
  local -r files="$1"

  local file=
  local realfile=
  local linkfile=
  while read -r file ; do
    if [ -n "${file}" ] ; then
      realfile="${INSTALL_DATA_DIR}/bin/${file}.sh"
      linkfile="${INSTALL_BIN_DIR}/${file}"

      if [ ! -f "${realfile}" ] ; then
        echofatal "'${realfile}' not found." >&2
        exit 101
      fi

      if [ -L "${linkfile}" ] ; then
        if [ "$( realpath "${linkfile}" )" = "${realfile}" ] ; then
          rm -v "${linkfile}"
        fi
      fi
      ln -sv "${realfile}" "${linkfile}" || exit $?
    fi
  done <<<"${files}"
}

function install_main {
  mkdir -pv "${INSTALL_DATA_DIR}" || exit $?
  rsync -av \
    --exclude='package' \
    --exclude='.git' \
    --exclude='.gitignore' \
    --exclude='.gitlab-ci.yml' \
    --exclude='samples' \
    "${PROJECT_HOME}/"* "${INSTALL_DATA_DIR}" || exit $?
  chown -Rv root:root "${INSTALL_DATA_DIR}" || exit $?

  find "${INSTALL_DATA_DIR}" -type f -exec chmod -v 444 {} \; || exit $?
  find "${INSTALL_DATA_DIR}" -type d -exec chmod -v 755 {} \; || exit $?

  install_set_chmod_for_list 555 "${INSTALL_EXE}" || return $?
  install_links "${INSTALL_LINKS}" || return $?

  echoinfo "$( realpath "$0" ) done." >&2
}

function exit_if_not_root {
  if [ "${EUID}" -ne 0 ] ; then
    echo 'This script must be run as root.' >&2
    exit 100
  fi
}

function remove_previous_version_before_install {
  local remove='false'

  if [ -f "${REMOVE_EXE}" ] ; then
    echoinfo "Using: '${REMOVE_EXE}' --force-remove"
    bash "${REMOVE_EXE}" --force-remove
    remove='true'
  elif is_variable_define_and_not_empty_safe 'REMOVE_OLD' ; then
    if [ -f "${REMOVE_OLD}" ] ; then
      echoinfo "Using: '${REMOVE_OLD}' --force-remove"
      bash "${REMOVE_OLD}" --force-remove
      remove='true'
    fi
  fi

  if [ "${remove}" = 'false' ] ; then
    echowarn "File not found: '${REMOVE_EXE}'" >&2
  fi
}

function remove_links {
  local -r files="$1"

  local file=
  local realfile=
  local linkfile=
  while read -r file ; do
    if [ -n "${file}" ] ; then
      realfile="${INSTALL_DATA_DIR}/bin/${file}.sh"
      linkfile="${INSTALL_BIN_DIR}/${file}"

      if [ -L "${linkfile}" ] ; then
        if [ "$( readlink "${linkfile}" )" = "${realfile}" ] || [ -f "$( readlink "${linkfile}" )" ] ; then
          rm -v "${linkfile}"
        fi
      fi
    fi
  done <<<"${files}"
}

function _remove_main_ {
  local -r force_remove="${0:-false}"

  exit_if_not_root

  local scriptpath=
  scriptpath="$( realpath "$0" )" || return $?

  local remove='false'
  if [ "${force_remove}" = 'false' ] ; then
    if [ "${REMOVE_EXE}" != "${scriptpath}" ] &&  [ -f "${REMOVE_EXE}" ] ; then
      echoinfo "Using: '${REMOVE_EXE}' --force-remove"
      bash "${REMOVE_EXE}" --force-remove
      remove='true'
    elif is_variable_define_and_not_empty_safe 'REMOVE_OLD' ; then
      if [ -f "${REMOVE_OLD}" ] ; then
        echoinfo "Using: '${REMOVE_OLD}' --force-remove"
        bash "${REMOVE_OLD}" --force-remove
        remove='true'
      fi
    fi
  fi

  if [ "${remove}" = 'false' ] ; then
    remove_links "${INSTALL_LINKS}"

    rm -fvr "${INSTALL_DATA_DIR}" || return $?
  fi

  echoinfo "${scriptpath} done." >&2
}

function _remove_main_usage_ {
  cat <<EOF
Usage:

  $( basename "${BASH_SOURCE[0]}" ) --help
  $( basename "${BASH_SOURCE[0]}" ) [--force-remove]

Options:
  -h,       --help  Display help message and exit.
            --force-remove
                    Use this script to remove, don't try to find previous version.

EOF
}


function remove_main {
  local param=
  local param_force_remove='false'

  while [ "$#" -gt 0 ] ; do
    param="$1"
    shift

    case "${param}" in
      '-h'|'--help')
        _remove_main_usage_
        exit 0
        ;;

      '--force-remove')
        param_force_remove='true'
        ;;

      *)
        echo "Unknown option: '${param}'" >&2
        _remove_main_usage_ >&2
        exit 1
        ;;
    esac
  done

  _remove_main_ "${param_force_remove}" || return $?
}

exit_if_variable_not_define_or_empty 'INSTALL_BIN_DIR'
exit_if_variable_not_define_or_empty 'INSTALL_DATA_DIR'
exit_if_variable_not_define_or_empty 'REMOVE_EXE'
