#!/bin/bash
#
# freebox-tools:bin/freebox-bash-api.sh
#
if [ "$( type -t echofatal )" != 'function' ] ; then
  echo 'https://git.cclaude.rocks/bash-apis/bash-common-api is required: Function echofatal not found.' >&2
  exit 100
fi

declare -gr FREEBOX_URL_NO_PROTO='mafreebox.freebox.fr'

declare _API_VERSION=
declare _API_BASE_URL=

declare _FREEBOX_PUBLIC_HTTPS='false'
declare _FREEBOX_PUBLIC_DOMAIN_URL=
declare _FREEBOX_PUBLIC_HTTPS_PORT=

declare _SESSION_TOKEN=

declare _NOT_TRUSTED_ALLOWED='false'
declare _HTTPS_ONLY='true'

_PRINT_ERROR_ONCE_HTTPS_NOT_WORKING_PROPERLY="$( mktemp )"
declare -gr _PRINT_ERROR_ONCE_HTTPS_NOT_WORKING_PROPERLY

function _check_success {
  if [ -z "$1" ] ; then
    echofatal 'Freebox non trouvée'
    return 1
  fi

  local value=
  value="$( jq -r '.success' <<<"$1" )"
  if [ "${value}" != 'true' ]; then
    echo "$( jq -r '.msg' <<<"$1" ): $( jq -r '.error_code'  <<<"$1" )" >&2
    return 1
  fi
  return 0
}

function _print_error_once_https_not_working_properly {
  local _done_=
  read -r _done_ <"${_PRINT_ERROR_ONCE_HTTPS_NOT_WORKING_PROPERLY}"
  if [ -n "${_done_}" ] ; then
    return 0
  fi

  cat <<EOF >&2
* Attention: Freebox non trouvée en HTTPS....

- Elle n'est peut-être pas accessible en ce moment (Redémarrage, pas sur le
  même réseau, ...)
EOF

  if [ "${_NOT_TRUSTED_ALLOWED}" != 'true' ] ; then
    cat <<EOF >&2
- Elle n'est pas configurée pour le https,
- Elle n'a pas de certificat valide:
  - Ajouter la clé à votre magasin de confiance (choix le plus sécurisé, pas simple),
  - Ou envisagez l'utilisation de l'option --insecure
EOF

  fi
  echo 'done' >"${_PRINT_ERROR_ONCE_HTTPS_NOT_WORKING_PROPERLY}"
}

function _print_error_once_enable_http {
  cat <<EOF >&2
  - Ou autoriser l'usage du http (non sécurisé) à l'aide de l'option: --allow-http (le moins sécurisé)
EOF
}

function freebox_local_only_api_version_json { # API
  _FREEBOX_PUBLIC_HTTPS='false'
  freebox_api_version_json
}

function freebox_api_version_json { # API
  _do_curl 'GET' "/api_version" ''
}

function freebox_api_init_api_access { # API
  _FREEBOX_PUBLIC_HTTPS="${1:-false}"
  _FREEBOX_PUBLIC_DOMAIN_URL="${2:-UNSET}"
  _FREEBOX_PUBLIC_HTTPS_PORT="${3:-UNSET}"

  local answer=
  answer="$( freebox_api_version_json )"
  _API_VERSION="$( jq -r '.api_version' <<<"${answer}" | sed 's/\..*//')"
  _API_BASE_URL="$( jq -r '.api_base_url' <<<"${answer}" )"
}

function _do_curl {
  local -r method_request="$1"
  local -r api_uri="$2"
  local -r data="$3"
  local -a curl_options=('-s')
  local -a options=('')
  local    url=
  local    answer=

  if [ "${_NOT_TRUSTED_ALLOWED}" = 'true' ] ; then
    curl_options+=('-k')
  fi

  case "${method_request}" in
    'GET' ) ;;
    'POST') options+=(-X 'POST' ) ;;
    'PUT') options+=(-X 'PUT' ) ;;
    *)
      echo "ERROR: Unsupported request method '${method_request}" >&2
      exit 1
      ;;
  esac

  [ -n "${_SESSION_TOKEN}" ] && options+=(-H "X-Fbx-App-Auth: ${_SESSION_TOKEN}")
  [ -n "${data}" ] && options+=(-d "${data}")

  # External https URL
  if [ "${_FREEBOX_PUBLIC_HTTPS}" = 'true' ] ; then
    url="https://${_FREEBOX_PUBLIC_DOMAIN_URL}:${_FREEBOX_PUBLIC_HTTPS_PORT}${api_uri}"
    answer="$(
      set +e
      curl "${curl_options[@]}" "${url}" "${options[@]}"
      return 0
      )"
  fi

  # Internal https URL
  if [ -z "${answer}" ] ; then
    url="https://${FREEBOX_URL_NO_PROTO}${api_uri}"
    answer="$(
      set +e
      curl "${curl_options[@]}" "${url}" "${options[@]}"
      return 0
      )"
  fi

  # Internal http URL
  if [ -z "${answer}" ] ; then
    _print_error_once_https_not_working_properly

    if [ "${_HTTPS_ONLY}" != 'true' ] ; then
      url="http://${FREEBOX_URL_NO_PROTO}${api_uri}"
      answer="$(
        set +e
        curl "${curl_options[@]}" "${url}" "${options[@]}"
        return 0
        )"
    else
      _print_error_once_enable_http
    fi
  fi

  echo "${answer}"
}

function _freebox_api_request {
  local -r method_request="$1"
  local -r api_url="$2"
  local -r data="$3"
  local -a options=('')

  if [ -z "${_API_BASE_URL}" ] ; then
    echo "Votre Freebox n'a pas été trouvée. (_API_BASE_URL not define)" >&2
    exit 1
  fi
  if [ -z "${_API_VERSION}" ] ; then
    echo 'ERROR. _API_VERSION not define.' >&2
    exit 1
  fi

  local answer=
  answer="$( _do_curl "${method_request}" "${_API_BASE_URL}v${_API_VERSION}/${api_url}" "${data}" )"

  _check_success "${answer}" || return 1
  echo "${answer}"
}

function freebox_api_get { # API
  local -r api_url="$1"
  local -r data="${2-}"
  local -i i=0
  local    result=

  # shellcheck disable=SC2034
  for i in {1..3} ; do
    if result="$( _freebox_api_request 'GET' "${api_url}" "${data}" )" ; then
      echo "${result}"
      return 0
    fi
    sleep 1
  done

  echofatal "GET Error: ${api_url} ${data}"
  return 1
}

function freebox_api_get_with_result_if_success { # API
  local -r api_url="$1"
  local -r data="${2-}"

  local json=
  json="$( freebox_api_get "${api_url}" "${data}" )"

  if [ -n "${json}" ] ; then
    if [ "$( jq '.success' <<<"${json}" )" = 'true' ] ; then
      jq -r '.result' <<<"${json}"
    else
      return 254
    fi
  else
    return 255
  fi
}

function freebox_api_post { # API
  local -r api_url="$1"

  _freebox_api_request 'POST' "${api_url}" ''
}

function freebox_api_put { # API
  local -r api_url="$1"
  local -r data="$2"

  _freebox_api_request 'PUT' "${api_url}" "${data}"
}

function freebox_api_login { # API
  local -r p_app_id="$1"
  local -r p_app_token="$2"

  local answer=
  answer="$( freebox_api_get 'login' )" || return 1

  local challenge=
  local password=

  challenge="$( jq -r '.result.challenge' <<<"${answer}" )"
  password="$( echo -n "${challenge}" | openssl dgst -sha1 -hmac "${p_app_token}" | sed  's/^(stdin)= //' )"

  answer="$( freebox_api_get '/login/session/' "{\"app_id\":\"${p_app_id}\", \"password\":\"${password}\" }" )" || return 1
  _SESSION_TOKEN="$( jq -r '.result.session_token' <<<"${answer}" )"
}

function freebox_api_authorize_application {
  local -r p_app_id="$1"
  local -r p_app_name="$2"
  local -r p_app_version="$3"
  local -r p_device_name="$4"

  local answer=
  answer="$(
    freebox_api_get \
        'login/authorize' \
        "{\"app_id\":\"${p_app_id}\", \"app_name\":\"${p_app_name}\", \"app_version\":\"${p_app_version}\", \"device_name\":\"${p_device_name}\" }"
    )"
  [ -z "${answer}" ] && exit 1

  local app_token=
  local track_id=

  app_token="$( jq -r '.result.app_token' <<<"${answer}" )"
  track_id="$( jq -r '.result.track_id' <<<"${answer}" )"

  cat <<EOF >&2

Autorisez l'application depuis l'écran LCD de la Freebox..."

EOF

  local status='pending'
  while [ "${status}" = 'pending' ] ; do
    sleep 5
    answer="$( freebox_api_get "login/authorize/${track_id}" )"
    status="$( jq -r '.result.status' <<<"${answer}" )"
  done

  echo "Etat de l'autorisation: ${status}" >&2
  [ "${status}" != 'granted' ] && return 1

  echo >&2
    cat <<EOF
new_app_id="${p_app_id}"
new_app_token="${app_token}"
EOF
}

function set_insecure { # API
  _NOT_TRUSTED_ALLOWED='true'
}

function set_allow_http { # API
  _HTTPS_ONLY='false'
}
