#!/bin/bash
#
# freebox-tools:bin/freebox-config-api.sh
#
# Manage local configuration
#
if [ "$( type -t is_variable_define_and_not_empty_safe )" != 'function' ] ; then
  echo 'https://git.cclaude.rocks/bash-apis/bash-common-api is required: Function is_variable_define_and_not_empty_safe not found.' >&2
  exit 1
fi

declare -gr FREEBOX_CONFIG_DIRECTORY="${HOME}/.config/freebox"

declare -gr FREEBOX_CONFIG_APP_ID="${FREEBOX_CONFIG_DIRECTORY}/appid"
declare -gr FREEBOX_CONFIG_APP_TOKEN="${FREEBOX_CONFIG_DIRECTORY}/apptoken"

declare -gr FREEBOX_CONFIG_PUBLIC_HTTPS="${FREEBOX_CONFIG_DIRECTORY}/https"
declare -gr FREEBOX_CONFIG_PUBLIC_DOMAIN_URL="${FREEBOX_CONFIG_DIRECTORY}/domain"
declare -gr FREEBOX_CONFIG_PUBLIC_HTTPS_PORT="${FREEBOX_CONFIG_DIRECTORY}/port"

declare -gr _APP_ID_='freebox-tools'
declare -gr _APP_NAME_='FreeBox Toolbox (bash)'
declare -gr _APP_VERSION_='1.0.4'

#
# Verify configuration
#
function _freebox_config_check {
  if ! is_variable_define_and_not_empty_safe MY_FREEBOX_APP_ID ; then
    echo "MY_FREEBOX_APP_ID is not define"
    exit 1
  fi

  if ! is_variable_define_and_not_empty_safe MY_FREEBOX_APP_TOKEN ; then
    echo "MY_FREEBOX_APP_TOKEN is not define"
    exit 1
  fi
}

#
# Save configuration
#

function _save_token_from_freebox {
  local -r MY_FREEBOX_APP_ID="$1"
  local -r MY_FREEBOX_APP_TOKEN="$2"

  _freebox_config_check

  mkdir -vp "${FREEBOX_CONFIG_DIRECTORY}" || exit 1

  echo "${MY_FREEBOX_APP_ID}" >"${FREEBOX_CONFIG_APP_ID}"       || exit 1
  echo "${MY_FREEBOX_APP_TOKEN}" >"${FREEBOX_CONFIG_APP_TOKEN}" || exit 1
}

function freebox_config_init_token {
  local new_app_id=
  local new_app_token=

  local result=
  result="$( mktemp )" || return $?

  # fill _API_VERSION and _API_BASE_URL variables
  freebox_api_init_api_access

  # Negociate new token
  freebox_api_authorize_application "${_APP_ID_}" "${_APP_NAME_}" "${_APP_VERSION_}" "$( hostname )" >"${result}"

  # shellcheck disable=SC1090
  source "${result}" || return $?
  rm "$( mktemp )" || return $?

  if [ -n "${new_app_id}" ] && [ -n "${new_app_token}" ] ; then
    _save_token_from_freebox "${new_app_id}" "${new_app_token}" || return $?

    echo "${FREEBOX_CONFIG_DIRECTORY} updated."
  else
    echo 'Problème lors de la récupération du token.'
    return 100
  fi

  local answer=
  answer="$( freebox_local_only_api_version_json )" || return $?

  local api_public_domain_url=
  local api_public_https=
  local api_public_https_port=

  api_public_https="$( jq -r '.https_available' <<<"${answer}" )" || return $?
  api_public_domain_url="$( jq -r '.api_domain' <<<"${answer}" )" || return $?
  api_public_https_port="$( jq -r '.https_port' <<<"${answer}" )" || return $?

  echo "${api_public_https}"      >"${FREEBOX_CONFIG_PUBLIC_HTTPS}"      || return $?
  echo "${api_public_domain_url}" >"${FREEBOX_CONFIG_PUBLIC_DOMAIN_URL}" || return $?
  echo "${api_public_https_port}" >"${FREEBOX_CONFIG_PUBLIC_HTTPS_PORT}" || return $?
}

#
# Load configuration
#
function _freebox_config_load {
  # Mandatory configuration
  declare -g MY_FREEBOX_APP_ID=
  declare -g MY_FREEBOX_APP_TOKEN=

  if [ -f "${FREEBOX_CONFIG_APP_ID}" ] ; then
    read -r MY_FREEBOX_APP_ID < "${FREEBOX_CONFIG_APP_ID}" || exit 1
  fi
  if [ -f "${FREEBOX_CONFIG_APP_TOKEN}" ] ; then
    read -r MY_FREEBOX_APP_TOKEN < "${FREEBOX_CONFIG_APP_TOKEN}" || exit 1
  fi

  declare -gr MY_FREEBOX_APP_ID
  declare -gr MY_FREEBOX_APP_TOKEN

  # Optionnal configuration
  declare -g MY_FREEBOX_PUBLIC_HTTPS=
  declare -g MY_FREEBOX_PUBLIC_DOMAIN_URL=
  declare -g MY_FREEBOX_PUBLIC_HTTPS_PORT=

  if [ -f "${FREEBOX_CONFIG_PUBLIC_HTTPS}" ] ; then
    read -r MY_FREEBOX_PUBLIC_HTTPS < "${FREEBOX_CONFIG_PUBLIC_HTTPS}" || exit 1
  else
    MY_FREEBOX_PUBLIC_HTTPS='false'
  fi
  if [ -f "${FREEBOX_CONFIG_PUBLIC_DOMAIN_URL}" ] ; then
    read -r MY_FREEBOX_PUBLIC_DOMAIN_URL < "${FREEBOX_CONFIG_PUBLIC_DOMAIN_URL}" || exit 1
  fi
  if [ -f "${FREEBOX_CONFIG_PUBLIC_HTTPS_PORT}" ] ; then
    read -r MY_FREEBOX_PUBLIC_HTTPS_PORT < "${FREEBOX_CONFIG_PUBLIC_HTTPS_PORT}" || exit 1
  fi

  declare -gr MY_FREEBOX_PUBLIC_HTTPS
  declare -gr MY_FREEBOX_PUBLIC_DOMAIN_URL
  declare -gr MY_FREEBOX_PUBLIC_HTTPS_PORT
}

#
# Freebox load configuration and login
#

function freebox_connect {
  _freebox_config_load
  _freebox_config_check

  # fill _API_VERSION and _API_BASE_URL variables
  freebox_api_init_api_access "${MY_FREEBOX_PUBLIC_HTTPS}" "${MY_FREEBOX_PUBLIC_DOMAIN_URL}" "${MY_FREEBOX_PUBLIC_HTTPS_PORT}"

  # login
  freebox_api_login "${MY_FREEBOX_APP_ID}" "${MY_FREEBOX_APP_TOKEN}"
}

